#!/usr/bin/env python

# This script converts a .schema file into, optionally, SQL commands,
# an HTML page and a text representation

import os, sys, re

# Class which wraps file open, write and close
# but only actually does them if output is enabled.

class OptionalFile:

   def __init__(self, filename, output_enabled):
       self.enabled = output_enabled
       if self.enabled == 1:
          if filename == None:
             self.file = sys.stdout
          else:
             self.file = open(filename, "w")

   def write(self, line):
       if self.enabled == 1:
           self.file.write(line)

   def close(self):
       if self.enabled == 1:
           self.file.close()

# Main script starts here

# Parse command line arguments

version = "5.0.31"
output_sql = 0
output_html = 0
output_clean = 0
output_file = None
showhelp = 0
showversion = 0

files = []
args = sys.argv[1:]
previous = ""

for arg in args:

    if previous == "-o":
        output_file = arg.strip()
    elif arg == "--help":
        showhelp = 1
    elif arg == "-v" or arg == "--version":
        showversion = 1
    elif arg == "-s" or arg == "--sql":
        output_sql = 1
    elif arg == "-h" or arg == "--html":
        output_html = 1
    elif arg == "-c" or arg == "--clean":
        output_clean = 1
    elif arg != "-o":
        files.append(arg)

    previous = arg
    
if showhelp == 1:
    print "Usage: new-rgma-schema-tool [options] [schema-files]"
    print "Options:"
    print "   --help         Display usage information"
    print "   -v, --version  Display version"
    print "   -s, --sql      Output SQL commands for creating tables"
    print "   -h, --html     Ouput HTML page showing table schema"
    print "   -c, --clean    Output text file showing table schema"
    print "   -o <file>      Output to a file rather than standard output"
    print ""
    print "Only one of -s, -h and -c may be specified"
    sys.exit(0)

if showversion == 1:
    print "new-rgma-schema-tool version " + version
    sys.exit(0)

total_out = output_sql + output_html + output_clean

if total_out == 0:
    print "ERROR: No output (SQL, HTML, clean) was requested"
    print "Try new-rgma-schema-tool --help for usage information"
    sys.exit(1)

if total_out > 1:
    print "ERROR: Only one of -s, -h and -c may be specified"
    print "Try new-rgma-schema-tool --help for usage information"
    sys.exit(1)
   
if len(files) == 0:
    print "ERROR: No schema files specified"
    print "Try new-rgma-schema-tool --help for usage information"
    sys.exit(1)

# Process schema files one at a time

for file in files:

    if not os.path.exists(file):
        print "ERROR: Schema file " + file + " not found"
        sys.exit(1)

    # Read the schema file
    
    f = open(file)
    lines = f.readlines()
    f.close

    # Parse the schema file to get the tables and column details
    
    table = re.compile("^table:\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.+)$",re.I)
    column = re.compile("(\*?)\s*(\S+)\s+(\S+)\s*(.*)$",re.I)
    
    i = 0
    bad = 0
    tname = ""
    tables = []
    columns = []

    for line in lines:
        i = i + 1
        line = line.strip()
        if line == "" or line[0:1] == "#": continue
        t = table.search(line)
        if t:
            if tname != "": tables.append((tkeep,columns))
            columns = []
            tname = t.group(1)
            tkeep = t.groups()
            continue
        c = column.search(line)
        if c and c.group(2).lower() != "table:":
            columns.append(c.groups())
            continue

        print "Bad line #", i, ":", line
        sys.exit(1)

    if tname != "": tables.append((tkeep,columns))    

    # Create files for the output that has been chosen

    clean = OptionalFile(output_file, output_clean)
    html = OptionalFile(output_file, output_html)
    sql = OptionalFile(output_file, output_sql)
    
    # Write HTML header

    base = file.replace(".schema", "")
    sname = os.path.basename(base)
    if sname[0].islower:  sname = sname[0].upper() + sname[1:]
    html.write("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\"><html><head><title>" + \
               sname + " R-GMA schema</title></head><body><h2>" + \
               sname +  " R-GMA schema</h2>")

    # Select schema database for SQL

    sql.write("use schema;")

    # Write table information to the output files
    
    maxcol = 0
    maxtab = 0
    maxkey = 0
    for table in tables:
        table, columns = table
        maxtab = max(maxtab, len(table[0]))
        maxkey = max(maxkey, len(table[1])+len(table[2])+len(table[3])+2)
        
        for column in columns:
            maxcol = max(maxcol, len(column[1]))
            
    for table in tables:
        table, columns = table

        clean.write("\n\nTable: " + table[0].ljust(maxtab+5) + (table[1] + " " +  table[2] + " " + table[3]).ljust(maxkey+5) + table[4])

        # Write SQL to delete existing schema for this table

        sql.write("\nSET @tableId = NULL;")
        sql.write("\nSELECT @tableId:=tableId FROM producerTables WHERE name='" + table[0] + "';")
        sql.write("\nDELETE FROM tableColumns WHERE tableId=@tableId;")
        sql.write("\nDELETE FROM producerTables WHERE name='" + table[0] + "';")
        
        # Write SQL to create new schema for this table
        
        sql.write("\n\nINSERT INTO producerTables VALUES(NULL, '" + table[0] +"', '', '" + table[4] + "', '" +table[1] + "', '" +table[2] + "', '" +table[3] + "', 'http://localhost.localdomain:8080/R-GMA/SchemaServlet', 0);")
        sql.write("\nSELECT @tableId:=tableId FROM producerTables WHERE name='" + table[0] + "';")
        html.write("\n<h3>"+table[0]+"</h3><p>"+table[4]+"</p>")
        html.write("\n<table summary=\"none\">")
                
        for column in columns:
            if column[0] == "*":
                primary = "1"
                col="<b>" + column[1] + "</b>"
            else:
                primary = "0"
                col = column[1]

            clean.write("\n   " + column[0].ljust(2) + column[1].ljust(maxcol+2) + column[2].ljust(14) + column[3])
            sql.write("\nINSERT INTO tableColumns VALUES (NULL, @tableId, '" + column[1] + "', '" + column[2] + "', " + primary + ", '0', '" + column[3] + "', 'http://localhost.localdomain:8080/R-GMA/SchemaServlet',0);")
            html.write("<tr><td>&nbsp;"+col+"&nbsp;</td><td>&nbsp;"+column[2]+"&nbsp;</td><td>&nbsp;"+column[3]+"&nbsp;</td></tr>")

        useNewRgmaColumns = True
        
        if useNewRgmaColumns:
	        list = [('RgmaTS',  'TIMESTAMP',   'Timestamp.'), \
	                ('RgmaHost','VARCHAR(15)', 'Host address.'), \
	                ('RgmaLRT', 'TIMESTAMP',   'Latest retention time.'), \
	                ('RgmaUser','VARCHAR(255)','User DN.'), \
	                ('RgmaTUID','INTEGER',     'Tuple unique ID.')]
        else:
			list = [('MeasurementDate', 'DATE', 'Measurement date.'), \
	                ('MeasurementTime', 'TIME', 'Measurement time.')]
            
        for tuple in list:
	        sql.write("\nINSERT INTO tableColumns VALUES (NULL, @tableId, '" + tuple[0] + "', '" + tuple[1] + "', 0, 0,'" + tuple[2] + "','http://localhost.localdomain:8080/rgma-axis/SchemaService',0);")
        
        html.write("\n</table>")
            
    html.write("\n</body></html>")
                
    clean.close()
    html.close()
    sql.close()
