#!/usr/bin/env python

# new-rgma-server-setup.py
#
#This script configures an RGMA server
#
#It generates an rgma-server.conf file based on which services have been
#requested by command line options. Following this, it generates SQL commands
#to create the necessary databases and if a schema is being configured it
#appends further SQL commands to create schema tables from installed .schema
#files. Finally, if servicetool is installed it arranges for RGMA services
#and their status to be periodically published.

import sys, os, socket, getopt

# Print usage information

def version():

    print"new-rgma-server-setup.py version 5.0.31"
    
def usage():

    print("Usage: new-rgma-server-setup.py [ --help ] [ --version ]")
    print("                                [ --produceruser=<user> ]")
    print("                                [ --producerpass=<password> ]")
    print("                                [ --schema=yes|no ]")
    print("                                [ --schemauser=<user> ]")
    print("                                [ --schemapass=<password> ]")
    print("                                [ --registry=yes|no ]")
    print("                                [ --registryuser=<user> ]")
    print("                                [ --registrypass=<password> ]")
    print("                                [ --browser=yes|no ]")
    print("                                [ --output=<filename> ]")
    print("")
    print("Options (defaults in brackets):")
    print("  --produceruser : Username to access the producer database (producer)")
    print("  --producerpass : Password to access the producer database (rgma)")
    print("  --schema   : Whether to run a schema service (no)")
    print("  --schemauser : Username to access the schema database (schema)")
    print("  --schemapass : Password to access the schema database (rgma)")
    print("  --registry : Whether to run a registry service (no)")
    print("  --registryuser : Username to access the registry database (registry)")
    print("  --registrypass : Password to access the registry database (rgma)")
    print("  --browser  : Whether to run a browser service (no)")
    print("  --output   : Ouput file for SQL commands - default rgma-setup-db.sql")
    
def yes_or_no(string):

    string = string.lower().strip()
    return string == "y" or string == "yes"

def write_header(conf_file):


    conf_file.write("<?xml version=\"1.0\"?>")
    conf_file.write("<RGMA-Server>\n\n")
    
def write_footer(conf_file):

    conf_file.write("</RGMA-Server>\n")
    
def write_server_config(conf_file, dbuser, dbpassword):

    conf_file.write("<Common>\n")
    conf_file.write("  <accessListReadPeriod>60000</accessListReadPeriod>\n")
    conf_file.write("  <connectionIdFile>WEB-INF/tmp/arch-con-id</connectionIdFile>\n")
    conf_file.write("  <minTermInterval>600</minTermInterval>\n")
    conf_file.write("  <maxTermInterval>129600</maxTermInterval>\n")
    conf_file.write("</Common>\n")
    conf_file.write("\n")
    conf_file.write("<Producer>\n")
    conf_file.write("  <defaultMaxBufferSize>50000</defaultMaxBufferSize>\n")
    conf_file.write("  <defaultMinRetentionPeriod>5</defaultMinRetentionPeriod>\n")
    conf_file.write("  <cleanupPeriodForMinRetentionTracker>30</cleanupPeriodForMinRetentionTracker>\n")
    conf_file.write("  <maxTupleMemory>22</maxTupleMemory>\n")
    conf_file.write("  <minMemoryNeededForInsert>0</minMemoryNeededForInsert>\n")
    conf_file.write("  <tableAlreadyExistsErrorCode>1050</tableAlreadyExistsErrorCode>\n")
    conf_file.write("  <dbDriver>org.gjt.mm.mysql.Driver</dbDriver>")
    conf_file.write("  <defaultDbLocation>jdbc:mysql://localhost:3306/ProducerDB_</defaultDbLocation>\n")
    conf_file.write("  <defaultDbUsername>%s</defaultDbUsername>\n" % dbuser)
    conf_file.write("  <defaultDbPassword>%s</defaultDbPassword>\n" % dbpassword)
    conf_file.write("  <dbAdminLocation>jdbc:mysql://localhost:3306/mysql</dbAdminLocation>\n")
    conf_file.write("  <dbAdminUsername>%s</dbAdminUsername>\n" % dbuser)
    conf_file.write("  <dbAdminPassword>%s</dbAdminPassword>\n" % dbpassword)
    conf_file.write("  <maxBlockSendSize>5000</maxBlockSendSize>\n")
    conf_file.write("  <physicalNamePrefix>_RGMA_</physicalNamePrefix>\n")    
    conf_file.write("</Producer>\n")
    conf_file.write("\n")
    conf_file.write("<Consumer>\n")
    conf_file.write("  <bufferSize>10000</bufferSize>\n")
    conf_file.write("  <streamingPort>8088</streamingPort>\n")
    conf_file.write("  <notifyInterval>10000</notifyInterval>\n")
    conf_file.write("  <defaultQueryTimeOutPeriod>30000</defaultQueryTimeOutPeriod>\n")
    conf_file.write("  <responseTesterTimeOutPeriod>30000</responseTesterTimeOutPeriod>\n")
    conf_file.write("</Consumer>\n")
    conf_file.write("\n")

def write_schema_config(conf_file, dbuser, dbpassword):
    conf_file.write("<Schema>\n")
    conf_file.write("  <dbDriver>org.gjt.mm.mysql.Driver</dbDriver>\n")
    conf_file.write("  <dbLocation>jdbc:mysql://localhost:3306/schema</dbLocation>\n")
    conf_file.write("  <dbUsername>%s</dbUsername>\n" % dbuser)
    conf_file.write("  <dbPassword>%s</dbPassword>\n" % dbpassword)
    conf_file.write("</Schema>\n")
    conf_file.write("\n")
    
def write_registry_config(conf_file, dbuser, dbpassword):
    conf_file.write("<Registry replicate=\"true\">\n")
    conf_file.write("  <dbDriver>org.gjt.mm.mysql.Driver</dbDriver>\n")
    conf_file.write("  <dbLocation>jdbc:mysql://localhost:3306/registry</dbLocation>\n")
    conf_file.write("  <dbUsername>%s</dbUsername>\n" % dbuser)
    conf_file.write("  <dbPassword>%s</dbPassword>\n" % dbpassword)
    conf_file.write("  <replicationPeriod>30000</replicationPeriod>\n")
    conf_file.write("  <notifyInterval>30000</notifyInterval>\n")
    conf_file.write("  <duplicateKeyErrorCode>1062</duplicateKeyErrorCode>\n")
    conf_file.write("</Registry>\n")
    conf_file.write("\n")
    
def write_browser_config(conf_file):
    conf_file.write("<Browser>\n")
    conf_file.write("</Browser>\n")
    conf_file.write("\n")
    
# Check RGMA_HOME is set

try:
    rgma_home = os.environ['RGMA_HOME']
except KeyError:
    try:
        rgma_home = os.environ['GLITE_LOCATION']
    except KeyError:
        print "ERROR: Environment variable RGMA_HOME or GLITE_LOCATION must be set"
        sys.exit(1)
 
print "Using RGMA_HOME=" + rgma_home
 
# Get hostname

hostname = socket.getfqdn();

# Parse command line options

known_options = ["help", "version", \
                 "schema=","registry=","browser=", "output=", \
                 "schemauser=", "schemapass=", \
                 "registryuser=", "registrypass=", \
                 "produceruser=", "producerpass="]
try:
    options, args = getopt.getopt(sys.argv[1:], "", known_options)
except getopt.GetoptError:
    print("ERROR: parsing command line options\n")
    usage()
    sys.exit(1)

for arg in args:
    print("WARNING: unrecognised command line option '%s' ignored" % arg)

schema = False
registry = False
browser = False
output_file = None

produceruser = "producer"
producerpass = "rgma"
schemauser = "schema"
schemapass = "rgma"
registryuser = "registry"
registrypass = "rgma"

for option, value in options:
    if option == "--help":
        usage()
        sys.exit(0)
    elif option == "--version":
        version()
        sys.exit(0)
    elif option == "--produceruser":
        produceruser = value
    elif option == "--producerpass":
        producerpass = value
    elif option == "--schema":
        schema = yes_or_no(value)
    elif option == "--schemauser":
        schemauser = value
    elif option == "--schemapass":
        schemapass = value
    elif option == "--registry":
        registry = yes_or_no(value)
    elif option == "--registryuser":
        registryuser = value
    elif option == "--registrypass":
        registrypass = value
    elif option == "--browser":
        browser = yes_or_no(value)
    elif option == "--output":
        output_file = value

# Welcome

print
print "Setting up an R-GMA Server..."
print

# Write config file

conf_filename = os.path.join(rgma_home, "etc/rgma-server/rgma-server.conf")
conf_file = open(conf_filename, "w")

write_header(conf_file)

print "  - Configuring Consumer and Producer services"
write_server_config(conf_file, produceruser, producerpass)

if schema:
    print "  - Configuring a Schema service"
    write_schema_config(conf_file, schemauser, schemapass)
else:
    print "  - Not configuring a Schema service"
    
if registry:
    print "  - Configuring a Registry service"
    write_registry_config(conf_file, registryuser, registrypass)
else:
    print "  - Not configuring a Registry service"

if browser:
    print "  - Configuring a Browser service"
    write_browser_config(conf_file)
else:
    print "  - Not configuring a Browser service"

write_footer(conf_file)
conf_file.close()

print
print "Wrote configuration to: %s" % conf_filename
print

# Generate SQL to setup databases

print "  - Creating SQL commands to set up databases"
print

sql_filename = os.path.join(rgma_home, "etc/rgma-server/rgma_sql_conf.sql")
args = " -o " + sql_filename

setup_db_script = os.path.join(rgma_home, "share/rgma/scripts/rgma-setup-db.py") + args
os.system(setup_db_script)

# Install schema files

if schema:

  temp_filename = os.path.join(rgma_home, "etc/rgma-server/rgma_temp.sql")
  schema_dir = os.path.join(rgma_home, "share/rgma/schemas")
  try:
    schema_files = os.listdir(schema_dir)
  except:
    schema_files = []
  
  if len(schema_files) > 0:

    print
    print "  - Copying %i schema files from %s" % (len(schema_files), schema_dir) + " to sql configuration file"
    print
    
    install_schema_script = os.path.join(rgma_home, "share/rgma/scripts/new-rgma-install-schema.py")

    os.system(install_schema_script + " -o " + temp_filename + " " + schema_dir + "/*")
    temp_file = open(temp_filename, "r")
    sql_file = open(sql_filename, "a")
    sql_lines = temp_file.readlines()
    
    for line in sql_lines:
        sql_file.write(line)
   
    sql_file.close()
    temp_file.close()
    os.remove(temp_filename)

# Configure rgma-servicetool/services.xml to include R-GMA services

servicetool = os.path.join(rgma_home, "etc/rgma-servicetool/services.xml")
service_tool_script = os.path.join(rgma_home, "share/rgma/scripts/rgma-servlet-status.py")
args = " -pa"

if os.path.exists(servicetool):
  print
  print  "  - Configuring the service publisher tool for R-GMA services"
  os.system(service_tool_script + args)
else:
  print
  print "In order to publish the existence and status of the R-GMA services please install"
  print "glite-rgma-servicetool"
  print "and then run"
  print service_tool_script + args

print
print
print "All done - Please initialise MySQL using the following command:"
print "    mysql -u root -p < " + sql_filename
print
print "Then restart the Tomcat server"
print
