#!/usr/bin/env python

# This script generates a list of SQL commands to
# create the databases needed by an RGMA server

import os, sys, socket, xml.sax.handler

# SAX parser to parse useful info out of the server config file
class rgmaServerConfParser(xml.sax.handler.ContentHandler):

    START=1
    COMMON=2
    CONSUMER=3
    PRODUCER=4
    SCHEMA=5
    REGISTRY=6
    BROWSER=7
    
    def __init__(self):
        self.mode = 0
        self.chars = ""
        self.schema = False
        self.schema_password = None
        self.schema_user = None
        self.registry = False
        self.reg_password = None
        self.reg_user = None
        self.prod_password = None
        self.prod_user = None
                
    def startDocument(self):
        self.mode = self.START

    def startElement(self, name, attrs):
        if name == "Common":
            self.mode = self.COMMON
        elif name == "Consumer":
            self.mode = self.CONSUMER
        elif name == "Producer":
            self.mode = self.PRODUCER
        elif name == "Schema":
            self.schema = True
            self.mode = self.SCHEMA
        elif name == "Registry":
            self.registry = True
            self.mode = self.REGISTRY
        elif name == "Browser":
            self.mode = self.BROWSER
            
        self.chars = ""

    def endElement(self, name):
        if self.mode == self.PRODUCER:
            if name == "defaultDbUsername":
                self.prod_user = self.chars.strip()
            if name == "defaultDbPassword":
                self.prod_password = self.chars.strip()
        elif self.mode == self.SCHEMA:
            if name == "dbUsername":
                self.schema_user = self.chars.strip()
            if name == "dbPassword":
                self.schema_password = self.chars.strip()
        elif self.mode == self.REGISTRY:
            if name == "dbUsername":
                self.reg_user = self.chars.strip()
            if name == "dbPassword":
                self.reg_password = self.chars.strip()
        
        self.chars = ""
        
    def characters(self, content):
        self.chars = self.chars + content
    
# Function to write SQL to create a database.
# Includes command to drop the database if it already exists.

def create_db(db, user, password, mysql):
  mysql.write("DROP DATABASE IF EXISTS " + db + ";\n")
  mysql.write("CREATE DATABASE " + db + ";\n")
  mysql.write("USE " + db + ";\n")

# Function to write SQL from a file.

def write_file_sql(filename, mysql):
  f = open(filename)
  lines = f.readlines()
  for line in lines:
    mysql.write(line)

###########################
# Main script starts here #
###########################

# Check RGMA_HOME is set

try:
    rgma_home = os.environ['RGMA_HOME']
except KeyError:
    try:
        rgma_home = os.environ['GLITE_LOCATION']
    except KeyError:
        print "ERROR: Environment variable RGMA_HOME or GLITE_LOCATION must be set"
        sys.exit(1)

# Get hostname

hostname = socket.getfqdn();

showhelp = False
showversion = False
user = "root"
password = ""
output_file = None

version = "5.0.31"

# Check command line options

args = sys.argv[1:]
previous = ""

for arg in args:
    
    if previous == "-o":
        output_file = arg.strip()
    elif previous == "-u" or previous == "--user":
        user = arg.strip()
    elif previous == "-p" or previous == "--password":
        password = arg.strip()
    elif arg == "-h" or arg == "--help":
        showhelp = 1
    elif arg == "-v" or arg == "--version":
        showversion = 1

    previous = arg

# Show help

if showhelp == 1:
  print "'rgma-setup-db' sets up the basic RGMA database tables to operate a registry and/or schema"
  print "Usage: rgam-setup-db (options)"
  print ""
  print "Options:"
  print "    -h, --help                Display help information"
  print "    -v, --version             Display version"
  print "    -u, --user <user>         Database username to use for the setup (default: root)"
  print "    -p, --password <password> Database password to use for the setup (default: no password)"
  print "    -o <file>                 Write output to file instead of directly to MySQL"
  sys.exit(0)

# Show version

if showversion == 1:
  print "rgma-setup-db Version " + version
  sys.exit(0)

# Check rgma-server.conf to see what needs to be installed and
# what usernames / passwords to use

rgmaServerConf = os.path.join(rgma_home, "etc/rgma-server/rgma-server.conf")
parser = rgmaServerConfParser()
xml.sax.parse(rgmaServerConf, parser)

schema = parser.schema
schema_user = parser.schema_user
schema_password = parser.schema_password

registry = parser.registry
reg_user = parser.reg_user
reg_password = parser.reg_password

prod_user = parser.prod_user
prod_password = parser.prod_password

# Connect to the database or open the output file

if output_file == None:
    (mysql, stdout) = os.popen2("mysql -u " + user + " --pass=" + password, "w")
    completion_message = "Installed"
    start_message = "Installing"
else:
    mysql = open(output_file, "w")
    completion_message = "Generating SQL for"
    start_message = "Generated SQL for"
    
# Configure access permissions for databases

mysql.write("use mysql;\n")

if schema:

  print start_message + " users and permissions for Schema database"
  
  mysql.write("delete from user where user like '" + schema_user + "';\n")
  mysql.write("delete from db where user like '" + schema_user + "';\n")
  if not hostname == "localhost":
    mysql.write("insert into user (host,user,password) values ('localhost','" + schema_user + "',password('" + schema_password + "'));\n")
  if not hostname == "127.0.0.1":
    mysql.write("insert into user (host,user,password) values ('127.0.0.1','" + schema_user + "',password('" + schema_password + "'));\n")
  if not hostname == "localhost.localdomain":
    mysql.write("insert into user (host,user,password) values ('localhost.localdomain','" + schema_user + "',password('" + schema_password + "'));\n")
  mysql.write("insert into user (host,user,password) values ('" + hostname + "','" + schema_user + "',password('" + schema_password + "'));\n")
  mysql.write("insert into db (host,db,user,select_priv,insert_priv,update_priv,delete_priv,create_priv,drop_priv) values ('%','schema','" + schema_user + "','Y','Y','Y','Y','Y','Y');\n")

if registry:

  print start_message + " users and permissions for Registry database"
  
  mysql.write("delete from user where user like '" + reg_user + "';\n") 
  mysql.write("delete from db where user like '" + reg_user + "';\n")
  if not hostname == "localhost":
    mysql.write("insert into user (host,user,password) values ('localhost','" + reg_user + "',password('" + reg_password + "'));\n")
  if not hostname == "127.0.0.1":
    mysql.write("insert into user (host,user,password) values ('127.0.0.1','" + reg_user + "',password('" + reg_password + "'));\n")
  if not hostname == "localhost.localdomain":
    mysql.write("insert into user (host,user,password) values ('localhost.localdomain','" + reg_user + "',password('" + reg_password + "'));\n") 
  mysql.write("insert into user (host,user,password) values ('" + hostname + "','" + reg_user + "',password('" + reg_password + "'));\n")
  mysql.write("insert into db (host,db,user,select_priv,insert_priv,update_priv,delete_priv,create_priv,drop_priv) values ('%','registry','" + reg_user + "','Y','Y','Y','Y','Y','Y');\n")

# Set up access permissions so that Producers can create their databases

print start_message + " users and permissions for Producer databases"

mysql.write("delete from user where user like '" + prod_user + "';\n") 
mysql.write("delete from db where user like '" + prod_user + "';\n")
if not hostname == "localhost":
  mysql.write("insert into user (host,user,password,create_priv,drop_priv) values ('localhost','" + prod_user + "',password('" + prod_password + "'),'Y','Y');\n")
if not hostname == "127.0.0.1":
  mysql.write("insert into user (host,user,password,create_priv,drop_priv) values ('127.0.0.1','" + prod_user + "',password('" + prod_password + "'),'Y','Y');\n")
if not hostname == "localhost.localdomain":
  mysql.write("insert into user (host,user,password,create_priv,drop_priv) values ('localhost.localdomain','" + prod_user + "',password('" + prod_password + "'),'Y','Y');\n") 
mysql.write("insert into user (host,user,password,create_priv,drop_priv) values ('" + hostname + "','" + prod_user + "',password('" + prod_password + "'),'Y','Y');\n")
mysql.write("insert into db (host,db,user,select_priv,insert_priv,update_priv,delete_priv,create_priv, drop_priv) values ('%','ProducerDB\_%','" + prod_user + "','Y','Y','Y','Y','Y', 'Y');\n")

mysql.write("flush privileges;\n")

# Create registry and schema database tables

if schema:

  print start_message + " Schema tables"

  create_db("schema", schema_user, schema_password, mysql)
  sql_file = os.path.join(rgma_home, "share/rgma/scripts/init-rgma-schema-tables.sql")
  write_file_sql(sql_file, mysql)

if registry:

  print start_message + " Registry tables"

  create_db("registry", reg_user, reg_password, mysql)
  sql_file = os.path.join(rgma_home, "share/rgma/scripts/init-rgma-registry-tables.sql")
  write_file_sql(sql_file, mysql)

status = mysql.close()

if status != None:
    print "ERROR: failed to execute SQL for database setup"
    sys.exit(1)
