#!/usr/bin/env python

# This script does useful stuff with .schema files
#
# For example:
#
#   * Converts into HTML
#   * Converts into text
#   * Converts into SQL commands to create the table
#   * Installs into MySQL

import os, sys, re, getopt, xml.sax.handler

class OptionalFile:
    """
    Wraps the usual file open, write and close methods
    but only actually does them if output is enabled
    """

    def __init__(self, filename, output_enabled):
        self.enabled = output_enabled
        if self.enabled == 1:
            if filename == None:
                self.file = sys.stdout
            else:
                self.file = open(filename, "w")

    def write(self, line):
        if self.enabled == 1:
            self.file.write(line)

    def close(self):
        if self.enabled == 1:
            self.file.close()

class rgmaServerConfParser(xml.sax.handler.ContentHandler):
    """
    Parse the rgma server config file to extract DB details for the schema
    """
    
    NOT_SCHEMA=0
    SCHEMA=1
    
    def __init__(self):
        self.mode = 0
        self.chars = ""
        self.schema_db_host = None
        self.schema_db_port = None
        self.schema_db_name = None
                
    def startDocument(self):
        self.mode = self.NOT_SCHEMA

    def startElement(self, name, attrs):
        if name == "Schema":
            self.mode = self.SCHEMA#
            
        self.chars = ""

    def endElement(self, name):
        if name == "dbLocation" and self.mode == self.SCHEMA:
            self.schema_db_host, self.schema_db_port, self.schema_db_name = self.get_db_details(self.chars.strip())
        elif name == "Schema":
            self.mode = self.NOT_SCHEMA

        self.chars = ""

    def characters(self, content):
        self.chars = self.chars + content


    def get_db_details(self, url):
        """
        Get the database host, port and name JDBC URL string
        """
        dbstart = url.find("//") + 2
        if dbstart == 1:
            dbstart = 0
            
        db = url[dbstart:]
        host = db[:db.find(":")]
        port = db[db.find(":")+1:db.find("/")]
        name = db[db.find("/") + 1:]
        return (host, port, name)

def usage():
    sys.stderr.write("rgma-schema-tool - Process R-GMA .schema files\n\n")
    sys.stderr.write("Usage: rgma-schema-tool [options] [schema-files]\n")
    sys.stderr.write("Options:\n")
    sys.stderr.write("   --help                    Display usage information\n")
    sys.stderr.write("   -v, --version             Display version\n")
    sys.stderr.write("   -s, --sql                 Output SQL commands for creating tables\n")
    sys.stderr.write("   -i, --install             Install tables into MySQL Schema database\n")
    sys.stderr.write("   -h, --html                Ouput HTML page showing table schema\n")
    sys.stderr.write("   -t, --text                Output text file showing table schema\n")
    sys.stderr.write("   -u, --user=<user>         Database username for the installation (default: root).\n")
    sys.stderr.write("                             Only relevant when using -i option\n")
    sys.stderr.write("   -p, --password=<password> Database password for the installation\n")
    sys.stderr.write("                             (default: no password). Only relevant when using -i option\n")
    sys.stderr.write("   -o, --output=<file>       Output to <file> rather than standard output. Only relevant\n")
    sys.stderr.write("                             when used with -s -h or -t options\n")
    sys.stderr.write("")
    sys.stderr.write( "Only one of -s, -i, -h and -t may be specified\n")

def parse_schema(lines):
    """
    Parse the schema file to get the tables and column details
    """
    
    table = re.compile("^table:\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.+)$",re.I)
    column = re.compile("(\*?)\s*(\S+)\s+(\S+)\s*(.*)$",re.I)
    
    i = 0
    bad = 0
    tname = ""
    tables = []
    columns = []

    for line in lines:
        i = i + 1
        line = line.strip()
        if line == "" or line[0:1] == "#": continue
        t = table.search(line)
        if t:
            if tname != "": tables.append((tkeep,columns))
            columns = []
            tname = t.group(1)
            tkeep = t.groups()
            continue
        c = column.search(line)
        if c and c.group(2).lower() != "table:":
            columns.append(c.groups())
            continue

        sys.stderr.write("Bad line #%i:%s" % (i, line))
        sys.exit(1)

    if tname != "": tables.append((tkeep,columns))    
    return tables

def show_version():
    print "rgma-schema-tool version 5.0.31"

def write_text(file, table_details, columns):
    """
    Write table as plain text
    """
    file.write("\nTable: " + table_details[0].ljust(maxtab+5) + (table_details[1] + " " +  table_details[2] + " " + table_details[3]).ljust(maxkey+5) + table_details[4])
                
    for column in columns:
        file.write("\n   " + column[0].ljust(2) + column[1].ljust(maxcol+2) + column[2].ljust(14) + column[3])

    file.write("\n")

def write_html(file, table_details, columns):
    """
    Write table as HTML
    """
    file.write("\n<h3>"+table_details[0]+"</h3><p>"+table_details[4]+"</p>")
    file.write("\n<table summary=\"none\">")
                
    for column in columns:
        if column[0] == "*":
            col="<b>" + column[1] + "</b>"
        else:
            col = column[1]

        file.write("<tr><td>&nbsp;"+col+"&nbsp;</td><td>&nbsp;"+column[2]+"&nbsp;</td><td>&nbsp;"+column[3]+"&nbsp;</td></tr>")

    file.write("\n</table>")
        
def write_sql(file, table_details, columns):
    """
    Write SQL commands to create table in Schema
    """
    
    # Delete existing schema for this table

    file.write("\nSET @tableId = NULL;")
    file.write("\nSELECT @tableId:=tableId FROM producerTables WHERE name='" + table_details[0] + "';")
    file.write("\nDELETE FROM tableColumns WHERE tableId=@tableId;")
    file.write("\nDELETE FROM producerTables WHERE name='" + table_details[0] + "';")
        
    # Create new schema for this table
        
    file.write("\n\nINSERT INTO producerTables VALUES(NULL, '" + table_details[0] +"', '', '" + table_details[4] + "', '" +table_details[1] + "', '" +table_details[2] + "', '" +table_details[3] + "', 'http://localhost.localdomain:8080/R-GMA/SchemaServlet', 0);")
    file.write("\nSELECT @tableId:=tableId FROM producerTables WHERE name='" + table_details[0] + "';")
                
    for column in columns:
        if column[0] == "*":
            primary = "1"
        else:
            primary = "0"
            
        file.write("\nINSERT INTO tableColumns VALUES (NULL, @tableId, '" + column[1] + "', '" + column[2] + "', " + primary + ", '0', '" + column[3] + "', 'http://localhost.localdomain:8080/R-GMA/SchemaServlet',0);")
        
    useNewRgmaColumns = False
        
    if useNewRgmaColumns:
        list = [('RgmaTS',  'TIMESTAMP',   'Timestamp.'), \
                ('RgmaHost','VARCHAR(15)', 'Host address.'), \
                ('RgmaLRT', 'TIMESTAMP',   'Latest retention time.'), \
                ('RgmaUser','VARCHAR(255)','User DN.'), \
                ('RgmaTUID','INTEGER',     'Tuple unique ID.')]
    else:
        list = [('MeasurementDate', 'DATE', 'Measurement date.'), \
                ('MeasurementTime', 'TIME', 'Measurement time.')]
        
    for tuple in list:
        file.write("\nINSERT INTO tableColumns VALUES (NULL, @tableId, '" + tuple[0] + "', '" + tuple[1] + "', 0, 0,'" + tuple[2] + "','http://localhost.localdomain:8080/rgma-axis/SchemaService',0);")

    
###########################
# Main script starts here #
###########################

# Check RGMA_HOME is set

try:
    rgma_home = os.environ['RGMA_HOME']
except KeyError:
    try:
        rgma_home = os.environ['GLITE_LOCATION']
    except KeyError:
        print "ERROR: Environment variable RGMA_HOME or GLITE_LOCATION must be set"
        sys.exit(1)

# Parse command line arguments

sql = 0
html = 0
text = 0
install = 0

output_file = None
db_admin_user = "root"
db_admin_password = None
showhelp = 0
showversion = 0

files = []
previous = ""

# Parse command line options

short_options = "vsihto:"

long_options = ["help", "version",\
                "sql", "install", "html", "text", \
                "user=", "password=", "output="]
try:
    options, args = getopt.getopt(sys.argv[1:], short_options, long_options)
except getopt.GetoptError:
    sys.stderr.write("ERROR: parsing command line options\n")
    usage()
    sys.exit(1)

for option, value in options:

    if option == "--help":
        showhelp = 1
    if option == "-v" or option == "--version":
        showversion = 1
    elif option == "-s" or option == "--sql":
        sql = 1
    elif option == "-h" or option == "--html":
        html = 1
    elif option == "-t" or option == "--text":
        text = 1
    elif option == "-i" or option == "--install":
        install = 1
    elif option == "-u" or option == "--user":
        db_admin_user = value
    elif option == "-p" or option == "--password":
        db_admin_password = value
    elif option == "-o" or option == "--output":
        output_file = value

for arg in args:
    files.append(arg)
    
if showhelp == 1:
    usage()
    sys.exit(0)

if showversion == 1:
    show_version()
    sys.exit(0)

# Check the options specified are valid

total_modes = sql + html + text + install

if total_modes == 0:
    sys.stderr.write("ERROR: No output was requested - one of -s, -i, -h, and -t must be specified\n")
    sys.stderr.write("")
    usage()
    sys.exit(1)

if total_modes > 1:
    sys.stderr.write("ERROR: Only one of -s, -i, -h and -c may be specified\n")
    sys.stderr.write("")
    usage()
    sys.exit(1)
   
if len(files) == 0:
    sys.stderr.write("ERROR: No schema files specified\n")
    sys.stderr.write("")
    usage()
    sys.exit(1)

if install and output_file:
   sys.stderr.write("ERROR: Cannot specify an output file in schema install mode\n")
   sys.stderr.write("")
   usage()
   sys.exit(1)

# Get the name and location of the schema DB from the server config file

rgmaServerConf = os.path.join(rgma_home, "etc/rgma-server/rgma-server.conf")
parser = rgmaServerConfParser()
xml.sax.parse(rgmaServerConf, parser)

db_host = parser.schema_db_host
db_port = parser.schema_db_port
db_name = parser.schema_db_name

# Create files for the output that has been chosen

text_file = OptionalFile(output_file, text)
html_file = OptionalFile(output_file, html)

if install:
    if db_admin_password == None:
        sql_file, stdout = os.popen2("mysql --user=%s --host=%s --port=%s" % (db_admin_user, db_host, db_port))
    else:
        sql_file, stdout = os.popen2("mysql --user=%s --password=%s --host=%s --port=%s" % (db_admin_user, db_admin_password, db_host, db_port))
else:
    sql_file = OptionalFile(output_file, sql)

# Select schema database for SQL

sql_file.write("use %s;" % db_name)

# Write HTML header

html_file.write("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\">\n")
html_file.write("<html><head><title>R-GMA schemas</title></head>\n")
html_file.write("<body>\n")

# Process schema files one at a time

for file in files:

    if not os.path.exists(file):
        sys.stderr.write("ERROR: Schema file " + file + " not found\n")
        sys.exit(1)

    without_extension = file.replace(".schema", "")
    base_name = os.path.basename(without_extension)
    if base_name[0].islower:
        base_name = base_name[0].upper() + base_name[1:]

    # Read and parse the schema file
    
    f = open(file)
    lines = f.readlines()
    f.close

    tables = parse_schema(lines)

    # Write HTML file header

    html_file.write("<h2>Schemas from file %s</h2>\n" % base_name)

    # Write text file header

    text_file.write("*** Schemas from file %s ***\n\n" % base_name)
    
    # Write table information to the output files
    
    maxcol = 0
    maxtab = 0
    maxkey = 0
    
    for table in tables:
        table, columns = table
        maxtab = max(maxtab, len(table[0]))
        maxkey = max(maxkey, len(table[1])+len(table[2])+len(table[3])+2)
        
        for column in columns:
            maxcol = max(maxcol, len(column[1]))
            
    for table in tables:
        table_details, columns = table

        write_text(text_file, table_details, columns)
        write_html(html_file, table_details, columns)
        write_sql(sql_file, table_details, columns)

    # Write HTML footer
    
    html_file.write("\n</body></html>")
                
text_file.close()
html_file.close()
sql_file.close()
