#!/usr/bin/env python

# This script generates a list of SQL commands to
# create the databases needed by an RGMA server

import os, sys, socket, xml.sax.handler

# SAX parser to parse useful info out of the server config file
class rgmaServerConfParser(xml.sax.handler.ContentHandler):

    START=1
    COMMON=2
    CONSUMER=3
    PRODUCER=4
    SCHEMA=5
    REGISTRY=6
    BROWSER=7
    
    def __init__(self):
        self.mode = 0
        self.chars = ""
        self.schema = 0
        self.schema_db_password = None
        self.schema_db_host = None
        self.schema_db_port = None
        self.schema_db_name = None
        self.schema_db_user = None
        self.registry = 0
        self.reg_db_password = None
        self.reg_db_host = None
        self.reg_db_port = None
        self.reg_db_name = None
        self.reg_db_user = None
        self.monbox_db_host = None
        self.monbox_db_port = None
        self.monbox_db_name = None
        self.monbox_db_password = None
        self.monbox_db_user = None
                
    def startDocument(self):
        self.mode = self.START

    def startElement(self, name, attrs):
        if name == "Common":
            self.mode = self.COMMON
        elif name == "Consumer":
            self.mode = self.CONSUMER
        elif name == "Producer":
            self.mode = self.PRODUCER
        elif name == "Schema":
            self.schema = 1
            self.mode = self.SCHEMA
        elif name == "Registry":
            self.registry = 1
            self.mode = self.REGISTRY
        elif name == "Browser":
            self.mode = self.BROWSER
            
        self.chars = ""

    def endElement(self, name):
        if self.mode == self.PRODUCER:
            if name == "defaultDbUsername":
                self.monbox_db_user = self.chars.strip()
            if name == "defaultDbPassword":
                self.monbox_db_password = self.chars.strip()
            if name == "defaultDbLocation":
                self.monbox_db_host, self.monbox_db_port, self.monbox_db_name = self.get_db_details(self.chars.strip())
        elif self.mode == self.SCHEMA:
            if name == "dbUsername":
                self.schema_db_user = self.chars.strip()
            if name == "dbPassword":
                self.schema_db_password = self.chars.strip()
            if name == "dbLocation":
                self.schema_db_host, self.schema_db_port, self.schema_db_name = self.get_db_details(self.chars.strip())
        elif self.mode == self.REGISTRY:
            if name == "dbUsername":
                self.reg_db_user = self.chars.strip()
            if name == "dbPassword":
                self.reg_db_password = self.chars.strip()
            if name == "dbLocation":
                self.reg_db_host, self.reg_db_port, self.reg_db_name = self.get_db_details(self.chars.strip())
        
        self.chars = ""

    def characters(self, content):
        self.chars = self.chars + content

    def get_db_details(self, url):
        """
        Get the database host, port and name JDBC URL string
        """
        dbstart = url.find("//") + 2
        if dbstart == 1:
            dbstart = 0
            
        db = url[dbstart:]
        host = db[:db.find(":")]
        port = db[db.find(":")+1:db.find("/")]
        name = db[db.find("/") + 1:]
        return (host, port, name)

####################
# Helper functions #
####################

def create_db(db, mysql):
  """
  Write SQL to create a database, removing any currently-existing
  database of the same name
  """
  mysql.write("DROP DATABASE IF EXISTS " + db + ";\n")
  mysql.write("CREATE DATABASE " + db + ";\n")
  mysql.write("USE " + db + ";\n")

def write_file_sql(filename, mysql):
  """
  Write SQL from a file
  """
  f = open(filename)
  lines = f.readlines()
  for line in lines:
    mysql.write(line)

def setup_db_permissions(name, user, password, mysql, deleteUser=1):
    """
    Set up permissions for a user to access a database
    """
    mysql.write("use mysql;\n")
    mysql.write("delete from user where user like '" + user + "';\n") 
    if deleteUser:
        mysql.write("delete from db where user like '" + user + "';\n")

    hostname = socket.getfqdn()
    localhosts = ["localhost", "127.0.0.1", "localhost.localdomain"]
    if hostname not in localhosts:
        localhosts.append(hostname)
    for host in localhosts:
        mysql.write("insert into user (host,user,password,create_priv,drop_priv) values ('%s','%s',password('%s'),'Y','Y');\n" % (host, user, password))

    mysql.write("insert into db (host,db,user,select_priv,insert_priv,update_priv,delete_priv,create_priv, drop_priv, alter_priv, create_tmp_table_priv) values ('%%','%s','%s','Y','Y','Y','Y','Y','Y','Y', 'Y');\n" % (name, user))

    mysql.write("flush privileges;\n")

def show_version():
  print "rgma-setup-db version 5.0.31"

def show_help():
  print("Sets up the basic RGMA database tables to operate a monbox and/or registry and/or schema")
  print("Usage: rgma-setup-db (options)")
  print()
  print("Basic options: (defaults in brackets)")
  print("  -h, --help                     : Display help information")
  print("  -v. --version                  : Display version")
  print("  -o. --output <file>            : Output SQL commands for database setup to <file>")
  print("                                   ($RGMA_HOME/tmp/rgma-setup-db.sql)")
  print("  -i, --install                  : Install databases directly using the MySQL command line tool")
  print("  --db-admin-user <user>         : Admin user for database setup (root)")
  print("                                   Only relevant when using the --install option")
  print("  --db-admin-password <password> : Admin password for database setup (no password)")
  print("                                   Only relevant when using the --install option")
  print("  --silent                       : Do not write confirmation to standard output")
  print()
  print("Further Options:")
  print("  --monbox-admin-user <user>           : Admin user for installing the Monbox database")
  print("                                         Overrides the --db-admin-user option")
  print("                                         Only relevant when using the --install option")
  print("  --monbox-admin-password <password>   : Admin password for installing the Monbox database")
  print("                                         Overrides the --db-admin-password option")
  print("                                         Only relevant when using the --install option")
  print("  --schema-admin-user <user>           : Admin user for installing the Schema database")
  print("                                         Overrides the --db-admin-user option")
  print("                                         Only relevant when using the --install option")
  print("  --schema-admin-password <password>   : Admin password for installing the Schema database")
  print("                                         Overrides the --db-admin-password option")
  print("                                         Only relevant when using the --install option")
  print("  --registry-admin-user <user>         : Admin user for installing the Registry database")
  print("                                         Overrides the --db-admin-user option")
  print("                                         Only relevant when using the --install option")
  print("  --registry-admin-password <password> : Admin password for installing the Registry database")
  print("                                         Overrides the --db-admin-password option")
  print("                                         Only relevant when using the --install option")

###########################
# Main script starts here #
###########################

# Check RGMA_HOME is set

try:
    rgma_home = os.environ['RGMA_HOME']
except KeyError:
    try:
        rgma_home = os.environ['GLITE_LOCATION']
    except KeyError:
        print "ERROR: Environment variable RGMA_HOME or GLITE_LOCATION must be set"
        sys.exit(1)

# Get hostname

hostname = socket.getfqdn();

showhelp = 0
showversion = 0
silent = 0
admin_user = "root"
admin_password = ""
schema_db_admin_user = None
schema_db_admin_password = None
reg_db_admin_user = None
reg_db_admin_password = None
monbox_db_admin_user = None
monbox_db_admin_password = None

output_file = None
install_direct = 0

version = "5.0.31"

# Check command line options

args = sys.argv[1:]
previous = ""

for arg in args:
    
    if previous == "-o" or previous == "--output":
        output_file = arg.strip()
    elif previous == "-u" or previous == "--db-admin-user":
        admin_user = arg.strip()
    elif previous == "-p" or previous == "--db-admin-password":
        admin_password = arg.strip()
    elif previous == "--schema-admin-user":
        schema_db_admin_user = arg.strip()
    elif previous == "--schema-admin-password":
        schema_db_admin_password = arg.strip()
    elif previous == "--registry-admin-user":
        reg_db_admin_user = arg.strip()
    elif previous == "--registry-admin-password":
        reg_db_admin_password = arg.strip()
    elif previous == "--monbox-admin-user":
        monbox_db_admin_user = arg.strip()
    elif previous == "--monbox-admin-password":
        monbox_db_admin_password = arg.strip()
    elif arg == "-h" or arg == "--help":
        showhelp = 1
    elif arg == "-v" or arg == "--version":
        showversion = 1
    elif arg == "-i" or arg == "--install":
        install_direct = 1
    elif arg == "--silent":
        silent = 1

    previous = arg

if showhelp:
    show_help()
    sys.exit(0)
    
if showversion:
    show_version()
    sys.exit(0)

if schema_db_admin_user == None:
    schema_db_admin_user = admin_user
if schema_db_admin_password == None:
    schema_db_admin_password = admin_password

if reg_db_admin_user == None:
    reg_db_admin_user = admin_user
if reg_db_admin_password == None:
    reg_db_admin_password = admin_password

if monbox_db_admin_user == None:
    monbox_db_admin_user = admin_user
if monbox_db_admin_password == None:
    monbox_db_admin_password = admin_password

# Check rgma-server.conf to see what needs to be installed and
# what usernames / passwords to use

rgmaServerConf = os.path.join(rgma_home, "etc/rgma-server/rgma-server.conf")
parser = rgmaServerConfParser()
xml.sax.parse(rgmaServerConf, parser)

install_schema = parser.schema
schema_db_host = parser.schema_db_host
schema_db_port = parser.schema_db_port
schema_db_user = parser.schema_db_user
schema_db_password = parser.schema_db_password
schema_db_name = parser.schema_db_name

install_registry = parser.registry
reg_db_host = parser.reg_db_host
reg_db_port = parser.reg_db_port
reg_db_user = parser.reg_db_user
reg_db_password = parser.reg_db_password
reg_db_name = parser.reg_db_name

install_monbox = 1
monbox_db_host = parser.monbox_db_host
monbox_db_port = parser.monbox_db_port
monbox_db_user = parser.monbox_db_user
monbox_db_password = parser.monbox_db_password

# Open the output file if required

if install_direct:
    if not silent:
        print "Setting up R-GMA databases"
        print
    completion_message = "  - Installed"
    start_message = "  - Installing"
else:
    if not silent:
        print "  Setting up R-GMA databases"
        print
    completion_message = "  - Generating SQL to set up"
    start_message = "  - Generated SQL to set up"
    if output_file != None:
        mysql = open(output_file, "w")
    else:
        mysql = sys.stdout

# Configure databases

if install_schema:

    # Configure Schema database

    if install_direct:
        (mysql, stdout) = os.popen2("mysql --user=%s --password=%s --host=%s --port=%s" % (schema_db_admin_user, schema_db_admin_password, schema_db_host, schema_db_port), "w")
    else:
        mysql.write("##\n## These commands are intended for the MySQL database on %s, port %s\n##\n" %
                    (schema_db_host, schema_db_port))

    if not silent:
        print start_message + " users and permissions for Schema database"
  
    setup_db_permissions(schema_db_name, schema_db_user, schema_db_password, mysql)

    if not silent:
        print start_message + " Schema tables"

    create_db(schema_db_name, mysql)
    sql_file = os.path.join(rgma_home, "share/rgma-server/sql/init-rgma-schema-tables.sql")
    write_file_sql(sql_file, mysql)

    if install_direct:
        status = mysql.close()

        if status != None:
            sys.stderr.write("ERROR: failed to set up DB for Schema\n")
            sys.exit(1)

if install_registry:

    # Configure Registry database

    if install_direct:
        (mysql, stdout) = os.popen2("mysql --user=%s --password=%s --host=%s --port=%s" % (reg_db_admin_user, reg_db_admin_password, reg_db_host, reg_db_port), "w")
    else:
        mysql.write("##\n## These commands are intended for the MySQL database on %s, port %s\n##\n" %
                    (reg_db_host, reg_db_port))

    if not silent:
        print start_message + " users and permissions for Registry database"
  
    setup_db_permissions(reg_db_name, reg_db_user, reg_db_password, mysql)

    if not silent:
        print start_message + " Registry tables"

    create_db(reg_db_name, mysql)
    sql_file = os.path.join(rgma_home, "share/rgma-server/sql/init-rgma-registry-tables.sql")
    write_file_sql(sql_file, mysql)

    if install_direct:
        status = mysql.close()

        if status != None:
            sys.stderr.write("ERROR: failed to set up DB for Registry\n")
            sys.exit(1)

if install_monbox:

    # Configure Monbox database - NB we always do this!

    if install_direct:
        (mysql, stdout) = os.popen2("mysql --user=%s --password=%s --host=%s --port=%s" % (monbox_db_admin_user, monbox_db_admin_password, monbox_db_host, monbox_db_port), "w")
    else:
        mysql.write("##\n## These commands are intended for the MySQL database on %s, port %s\n##\n" %
                    (monbox_db_host, monbox_db_port))

    if not silent:
        print start_message + " users and permissions for Producer databases"

    setup_db_permissions('ProducerDB\_%', monbox_db_user, monbox_db_password, mysql)
    # Use the magic 'deleteUser' flag to avoid wiping the previous configuration
    setup_db_permissions('\_RGMA\_%', monbox_db_user, monbox_db_password, mysql, 0)

    if install_direct:
        status = mysql.close()

        if status != None:
            sys.stderr.write("ERROR: failed to set up DB for Monbox\n")
            sys.exit(1)

# Close output file if we are using one
               
if not install_direct:
    status = mysql.close()

    if status != None:
        sys.stderr.write("ERROR: failed to write SQL for database setup to file %s\n" % output_file)
        sys.exit(1)
